########################################################
#                                                      #
#                   PSYC201 R Lab                      #
#       Lesson 12 - ANCOVA and repeated measures       #
#                                                      #
########################################################

library(PSYC201)

##############################
# 12.1 - ANCOVA              # 
##############################

# The good news is that you've learned everything you need to know to do ANCOVA
# All we need to do is learn how to interpret the output

# In this example from the PSYC201 package, we're testing how undergrads value their time
# So we give them a task that is either boring or exciting for 30 minutes, then give them $15
# Finally, we tell them they will have to continue the task for the remaining 30 minutes, but can buy their way out
# Thus we ask them how much of the $15 they would be willing to pay to get out of the task
# And lastly we measured their weekly income before they started
head(quit.payment)

# The first thing we might ask is: are undergrads willing to pay more to get out of a boring task?
# This would be a simple t-test or ANOVA:
t.test(Offer~Task,data=quit.payment)

# Well... the Boring group paid more than the Exciting group... but not much and it's not significant

# We can compare the data as well:
boxplot(Offer~Task,quit.payment)

# That data looks pretty noisy as compared to the difference in means though
# So maybe there is a signal here, it's just being drowned out by the noise...
# How can we do better?
# Well, one way is to just collect more data. But we don't have the luxury of doing that here
# Another way is to use ANCOVA

# In this case, we might believe that undergrads who make more money will value their time more, and thus make higher offers
# For our main question - is there an effect of task excitement on the offer - this is just noise
# But it's also explainable - and in doing so we can take out some noise that will make our desired signal more clear

# So - we should ask ourselves: is weekly income a good covariate?
# A good covariate will have two characteristics:
#  1) You should have a belief that it is linearly related to the dependent variable
#  2) It is independent of the other experimental manipulations

# For the first part, we may not be sure about linearity, but we do believe that it should be related, so that passes muster
# For the second part, it is unlikely that the excitement of the task would change how much money subjects are earning
# (A good way to ensure the second question is passed is to ask the covariate question(s) BEFORE experimental manipulation)

# To do ANCOVA in R, all we have to do is add income to the model
# Note though, WeeklyInc must be added as the first term (we'll see why later)
offer.cov = lm(Offer ~ WeeklyInc + Task, data=quit.payment)

# Then as a basis, we take the model with just the WeeklyInc variable
offer.base = lm(Offer ~ WeeklyInc, data=quit.payment)

# Now ANCOVA is just asking - is there any effect of the Task type above and beyond what is explained by weekly income?
# But we know how to do these sorts of model comparisions - with the anova() command
anova(offer.base,offer.cov)

# And now we have a significant result - it does seem as if people in the boring task will pay more to get away

# But there's an easier way to see this result... just run anova() on the full covariance model:
anova(offer.cov)

# You'll note that the Task term seems to match up perfectly with the above model comparison
# This is why I noted you should add the covariate first:
# Since R adds items to the model in a stepwise fashion, the Task term here is the effect of Task once WeeklyInc is already in the model
# If you do it the other way around, you get a different (incorrect) result:
anova(lm(Offer~Task+WeeklyInc,data=quit.payment))

# This is calculating the SS of Task before the covariate - but under ANCOVA you want to know what you can explain after the covariate
# And thus we get different (incorrect) test statistics.

# So why does the significance increase so much?
# We can see easily by plotting the data
# This requires going back to the original Plotting lesson where we learned how to scatterplot with categorical variables
with(quit.payment,plot(WeeklyInc,Offer,pch=as.integer(Task)))
abline(coef(offer.cov)[1],coef(offer.cov)[2])
abline(coef(offer.cov)[1] + coef(offer.cov)[3],coef(offer.cov)[2],lty=2)
legend('topleft',legend=levels(quit.payment$Task),pch=1:2,lty=1:2)

# What ANCOVA is doing is asking whether the lines are shifted relative to one another
# But the error noise is just the noise around the lines
# Although still a bit messy, you should be able to see that the error around the lines is less than the error without the lines

# Now what if you wanted to get an estimate of how much more someone in the Boring task is willing to pay to get out?
# Can we just take the differences of the means?
mean(subset(quit.payment,Task=='Boring')$Offer) - mean(subset(quit.payment,Task=='Exciting')$Offer)  # 0.9855

# Well - let's look at the coefficients of our model:
summary(offer.cov)

# The TaskExciting coefficient is -1.07 - which means that exciting people pay a dollar and seven cents less according to this model
# This is different from the differences in the means - why?

# With ANCOVAs, you are asking what is the difference in the Offer based on Type *assuming WeeklyInc is the same*
# But even if there is no difference between WeeklyInc between the two Types in theory...
# (and there shouldn't be if we've done random assignment correctly)
# ... in practice there will be small differences
mean(subset(quit.payment,Task=='Boring')$WeeklyInc) # 260.35
mean(subset(quit.payment,Task=='Exciting')$WeeklyInc) # 268.5

# Now in reality this is a tiny difference
# But when comparing factors in an ANCOVA, you need to account for this, because it will cause a difference

# This is easy to do when you only have two factors, as in this case
# When you have more, however, it's easiest just to run contrasts and look at the 'Estimate'
# For instance:
contrast(offer.cov,c(1,-1))

# Note here that the Estimate is 1.07 - exactly as we got from our model

# And for the last bit of this section, we ask: why use '+' in ANCOVA models and not '*'?

# When you add an interaction to an ANCOVA model, you are bringing it beyond the bounds of ANCOVA
# When you do, you tell the model that the slope of the covariate regressed against the dependent variable
#  will chance as a function of the treatment group
# But this breaks rule #2: by definition the effect of the covariate is no longer independent of the experimental manipulations

# Note: it is perfectly acceptable to have interactions between multiple categorical terms, so long as they do not interact with continuous terms
# Thus if you were running a two factor study, it would be perfectly appropriate to cross the factors, if the covariate is not crossed
# For instance this formula would be fine:
DV ~ COV + IV1 * IV2

# But this would not be an ANCOVA:
DV ~ COV * IV1 * IV2

# This is a valid model (as we will discuss later), but not an ANCOVA

# However, you will often want to run an interaction to assure yourself that it DOESN'T exist
# If you find a significant effect, then the assumptions of ANCOVA don't hold
# And you'll need to start wondering about how the supposed covariate and the treatment interact...
anova(lm(Offer ~ WeeklyInc*Task,quit.payment))

# Another thing you don't want to see is dependence between the covariate and the independent variables of note
# To test for this, you can run an ANOVA to determine whether the covariate is predictable by all of your main factors
# Again, you don't want to see significance here
# If you do, you can't tell whether it's the difference in your treatment or the covariate that causes any experimental effects
anova(lm(WeeklyInc ~ Task, quit.payment))

rm(offer.base,offer.cov)

##############################
# 12.2 - ANCOVA caveats      # 
##############################

# For those of you paying attention to the replicability crisis, you may have seen ANCOVA getting slammed in some papers
# This has lead some journals to play down ANCOVA or ask for results without ANCOVA as well as with
# So you might be asking yourself: what should I do?

# What you should do is hard to say, but here are a couple things you should NOT do:

# 1) Pick and choose your covariates
# This is one of the original sins noted in the 'False Positive Psychology' paper
# They showed that just deciding whether to use a dummy 'gender' variable more than doubled false positive rates
# If you go through and only pick the covariates that help your case, you are capitalizing on chance
# So don't do it
# Instead, select your covariates beforehand, and use those and all of those - not a subset that seems to fit best

# 2) Covariates failed? Ditch 'em!
# This is a less extreme version of #1, but more insidious
# Rather than picking and choosing from a set of covariates, you might try the full set and get a null result
# Then since you took #1 to heart, you wouldn't be tempted to root through those covariates for just the ones that helped
# But you might think: why not just try the test without covariates at all? That's just a pure ANOVA, right?
# Therein lies the rub: if you have decent covariates, you basically double your false alarm rate by doing this
# If you have a real effect, it is extremely rare for it to not show up with covariates but show up without them
# (unless you believe the covariates are mediators - but that's another story entirely)
# So pick one before you run any analyses: use covariates or don't. But you shouldn't do both

# Thus a good rule of thumb is to have your analysis designed beforehand and run only that analysis
# Asking additional questions of your subjects so that you might have more covariates to try later is bad practice
# (However, asking those questions as CYAs for reviewer comments is fine - just be sure to note that this in your papers)

##############################
# 12.3 - Repeated measures   #
##############################

# Let's start with a simple repeated measures example: the wine dataset in PSYC201
wine

# In this dataset we have a wine tasting where six judges each gave a rating to four different wines
# We want to know - is there a difference between the wines?
# Well, we can look at the mean scores:
with(wine,tapply(Score,Wine,mean))

# Looks like Wine 3 is worse... but is it significant?
summary(aov(Score ~ Wine, wine))

# Doesn't look like it... why not?
# We might expect judges to differ in how strict they are with their ratings
# And in fact, we can look at this data in tabular format:
wine.alt

# You'll note that the judges differ wildly - some really like wine (Judge 1), and some seem to hate it (Judge 3)
# Note that this is a within-subject design, with the Judge as subjects
# With a simple design like this, you can just add Judge first and test whether Wine has a significant effect on Score
summary(aov(Score ~ Judge + Wine, wine))

# Now it does!
# But why?

# Note that now the SS for Wine is the same in both summaries, but the F score is higher with Judge in the model
# But the SSE is very different...
# Here, the Judge factor has soaked up a large portion of the SSE, at a cost of 5 df
# Therefore, the MSE is about a third as small, with a corresponding increase in F

# But how does this fit with what we know about blocking and repeated measures?
# Recall in the conceptual lesson, with blocking we wanted to separate the SSE into two sources:
# 1) Error due to the subjects, and 2) error around the subjects
# Any within-subjects factors should use error around the subjects for the F-statistic
# While between-subjects factors should use the subject error

# In this example, Wine is a within-subject factor
# Thus we don't care about the subject error, just the error within each Judge
# But by adding Judge as a factor, the Residual error is the error around the subjects

# There's another way of adding blocking terms to aov objects as well - through Error()
# You can add an Error() term to aov formulas to tell R what you want to use as a blocking factor
# If you are using a fully within-subject design, you can just put the blocking factor in the Error()
# For instance:
summary(aov(Score ~ Wine + Error(Judge),wine))

# Now note that this looks a bit different than the typical anova table
# This is because aov is now explicitly separating the error terms
# In the top field, you have the error around the Judge factor
# Because this is fully within-subjects, the only Sums of Squares is the Residuals
# These are the SS-Subject due to Judge

# In the bottom field, you have both Wine and Residuals
# Here Wine is the same Sums of Squares as before
# And Residuals is the same Sums of Squares once Judge is taken out
# Thus, for Wine, you see the same F as the ANOVA with Judge as another term

# Now, note that the way we've written the Error term was a shortcut
# Technically, that should be Error(Judge / Wine)
# In this case, the '/' operator means 'within'
# Which is telling R that the Wine factor is within-subject
# Thus you could write:
summary(aov(Score ~ Wine + Error(Judge / Wine),wine))

# But if you could get this same information by just adding Judge as a factor, why do we care?
# Because often you do need to tell R how to divvy up error
# In this case (and very simple cases) it didn't matter, but we will see how it does later

# Also, why are we using aov? Why not lm?
# Simply because lm doesn't support an Error term
# So if you're doing repeated measures, you need to use aov()
# (Or mle based models... but we'll learn about that later)

# So let's move onto a more complicated design...
# For this example, let's go back to the puzzles data
head(puzzles)

# Recall from earlier lessons that people got puzzles in three different types of conditions:
#  (1) Difficulty (easy vs difficult)
#  (2) Type of puzzle (visual vs word)
#  (3) Emotional state (happy vs sad)
# We then measured how long it took them to solve each puzzle, and wanted to know if solution time differed by condition

# We could visualize this data quickly as:
par(mfrow = c(1,2))
with(subset(puzzles,Emotion=='Happy'),interaction.plot(PuzzleType,Difficulty,Time,main='Happy'))
with(subset(puzzles,Emotion=='Sad'),interaction.plot(PuzzleType,Difficulty,Time,main='Sad'))
par(mfrow = c(1,1))

# But before we ignored one of the data fields: SubjID
# We didn't test 160 different subjects.. instead we gave 40 subjects different puzzle types
levels(puzzles$SubjID)

# Now - emotional state isn't easily switched, so each subject was assigned to either the happy or sad state
# (The between-subjects manipulation)
# But we could give each subject easy word, hard word, easy visual, and hard visual puzzle types (within-subjects manipulations)

##########################
# One between-subject factor

# Let's start with a very simple question (between subject factors): does the emotion affect solution time?
# The model for this is quite simple
summary(aov(Time ~ Emotion, data=puzzles))

# Well this is clearly significant... but we are also treating all observations as independent
# But some people might be better at solving puzzles than others, and we can account for this
# And if we're just looking at emotion, there will be multiple observations per person

# So we write the model using using SubjID in the error term
# We don't need to use any slash because there are no within-subject effects
summary(aov(Time ~ Emotion + Error(SubjID),data=puzzles))

# So why is this different from the original test we used?
# In the original test, we treated all observations as independent
# Thus we assumed testing a person four different times is like getting four different tests
# This is clearly not true

# Instead, when you have multiple observations, R will segment the error into within subject, and between subject
# The within subject error is just the natural variability that people will have from task to task
# The between subject error measures the differences across people
# When you build the model correctly, R understands that Emotion is a between subjects factor and treats it appropriately
# Note that this is identical to averaging peoples' times and running an ANOVA on that - the way Ed mentioned you should do it

##########################
# One within-subject factor

# Next we might ask, does the difficulty affect solution time?
# Here we can again try our naive model without subjects:
summary(aov(Time ~ Difficulty, data = puzzles))

# So there's certainly an effect...
# But again we're treating all observations as independent
# And this is an even worse assumption now, because subjects differ, and you get easy and hard puzzles for each subject
# Thus observations are clearly not independent

# But never fear: appropriate model building in R can save you!
# Here we DO use the slash, because Difficulty is within subjectss
summary(aov(Time ~ Difficulty + Error(SubjID / Difficulty), data = puzzles))

# Note here that our F-value has increased from 190 to 357...
# As we accounted for variability between subjects, we got more power

# But why did we have to tell R that Difficulty is nested in SubjID?
# What would happen if we didn't?
summary(aov(Time ~ Difficulty + Error(SubjID),data=puzzles))

# Note that we no longer see the SubjID:Difficulty error... and Difficulty is assigned to Within error
# If we ignore the nesting call, then R assumes you just want a single error term
# But we're still getting multiple observations of each difficulty type per person... (one visual, one word)
# If we don't include the nesting term, then R treats the observations as independent
# But they're not
# If we do include the term, R understands that you mean to say that people will vary even if you give them the same test multiple times
# And it averages over that appropriately

# So big note: Always build your aov error models appropriately to get the right answer!

######################################################
# One within-subject factor, one between-subject factor

# What if we ignore subject?
summary(aov(Time ~ Difficulty * Emotion, data=puzzles))

# Well, the interaction is not quite significant, but the other two clearly are...
# But we're ignoring the effect of subject here
# So let's include it

# However, it's a bit more tricky here... each subject was assigned to one of the two emotional states
# So SubjID is nested withing Emotion, and Difficulty is nested within SubjID
# Thus we are dealing with a mixed design

# Writing out the model is still easy...
# We just write the fixed factors as normal, then nest the subject effects appropriately
summary(aov(Time ~ Difficulty * Emotion + Error(SubjID / Difficulty), data=puzzles))

# Here we have the three same error levels as we saw above, but now three fixed items:
#  Difficulty, Emotion, and the interaction

# But note where they have been assigned:
#  Emotion has been assigned to the between-subject error (as it is a between-subject factor)
#  Difficulty and the interaction have been assigned to the interaction error appropriately

# So long as you build your model correctly, R will assign factors to their appropriate error

######################################################
# One within-subject factor, two between-subject factors

# And you can even get more complex...
# What about the full three-way ANOVA?
# We did this easily in Lesson 12
summary(aov(Time ~ PuzzleType * Emotion * Difficulty, data=puzzles))

# Again, we can build in Subject effects
# But note, now PuzzleType, Difficulty, and that interaction are all nested within SubjID
# So our Error term becomes a bit more complex
summary(aov(Time ~ PuzzleType * Emotion * Difficulty + Error(SubjID / (PuzzleType * Difficulty)), data=puzzles))

# Now there are all sorts of different types of errors to account for...
# Explaining why all of this happens is is very difficult to explain without the math
# But note that R assigns the factors appropriately to each error type if you've built the model correctly
# And so you just need to find the factor you care about in the table to determine whether it's correct (accounting for Subject)

# Finally, note that PSYC201 functions like get.p and get.stat work on these complex aov objects as well
full.mod = aov(Time ~ PuzzleType * Emotion * Difficulty + Error(SubjID / (PuzzleType * Difficulty)), data=puzzles)
get.p(full.mod)
get.stat(full.mod)

# However, also note that it collapses across all error types
# So be careful, and just pull it out when you know what you are grabbing for the homework

rm(full.mod)
