########################################################
#                                                      #
#                   PSYC201 R Lab                      #
#              Lesson 12X - Answer Key                 #
#                                                      #
########################################################

library(PSYC201)
library(plyr)
library(reshape2)
library(ggplot2)

#######################
# E-Prime (courtesy of Evan Carr)
#   http://vulstats.ucsd.edu/RAssignData/DataCleaning_EPrime.csv

# This experiment is measuring how people process assymmetric facial expresions
# People watched four different faces (avatars) that made various expressions:
#  1) Their emotions could be angry or happy
#  2) The emotion assymmetry could start on the left or right side of the face
#  3) The assymmetry started either 20 or 400ms into the video
# Thus there were effectively 32 different videos they could watch

# The experiment was split into two phases
# In phase 1, subjects saw each video three times, and each time would rate the face on EITHER intensity, authenticity, or trustworthiness
# This rating was a Likert scale (1-7) and was recorded along with the time it took to respond
# In phase 2, subjects saw each video once, and judged which emotion was being expressed and which side of the face it started on
# The program recorded whether they were right for each (0,1) and their reaction time

# (Hint: you may need to use the dcast() function for this data set)
# (Hint2: the 'anger' and 'angry' emotions should be the same)

# The main analysis is looking at how emotion, side of assymmetry, and time of assymmetry affect various ratings and reaction times
# So you should always include those factors in your analysis

# You'll need to answer the following questions:
# a) Is the speed (RT) with which people make the emotional judgment in part 2 correlated with any of the three
#    speeds of rating from part 1?
# b) I want to make sure that the emotional manipulation worked - do people rate angry faces as more intense?
# c) Are they faster to rate angry faces as intense than happy faces?

# Now let's take a look at the data
# I should also note that this data has be *slightly* cleaned - some extraneous columns have been deleted already
epbase = read.csv('http://vulstats.ucsd.edu/RAssignData/DataCleaning_EPrime.csv')

# Huh - so there are separate collumns for all of the phase 1 data, and phase 2 data...
# And of course they don't overlap

# So the easiest way of splitting these out is, well, spliting them up
# Then deleting the columns that aren't used in each phase
epp1 = subset(epbase, !is.na(phase1rating.RESP))
epp1 = epp1[c(2:4,14:20)]
epp2 = subset(epbase, !is.na(phase2ratingEmotion.ACC))
epp2 = epp2[2:12]

# Next we rename the columns to:
#  (a) make it nicer to read
#  (b) match up the conditions across phase 1 and phase 2
names(epp1)[4:10] = c('Avatar','Emotion','OASSide','OASTime','Rating','RateRT','Type')
names(epp2)[4:11] = c('Avatar','Emotion','OASSide','OASTime','EmotionAcc','EmotionRT','OASAcc','OASRT')

# Then we need to clean out the anger/angry emotions from epp2
epp2$Emotion = ifelse(epp2$Emotion=='anger','angry',as.character(epp2$Emotion))

# And the avatar 0s
epp2 = subset(epp2,Avatar >= 1)

# Now phase 1 is three times as long as phase 2... since it has separate lines for each of the three judgments
# So we need to melt and recast this to put this into wide form to have 32 rows per subject with each condition on a line
epp1long = melt(epp1,id.vars=c('Subject','Age','Sex','Avatar','Emotion','OASSide','OASTime','Type'),)

# Then we need to recast this using the dcast() function
epp1wide = dcast(epp1long, Subject+Age+Sex+Avatar+Emotion+OASSide+OASTime ~ Type+variable)

# And then merging is exceptionally simple
epdat = merge(epp1wide,epp2,all=T)

# But we're not done yet...
# Never forget to coerce numeric factors into the factors they should be!
epdat$Subject = factor(epdat$Subject)
epdat$Avatar = factor(epdat$Avatar)
epdat$OASTime = factor(epdat$OASTime)

# Now let's get to the questions
# a) Is the speed (RT) with which people make the emotional judgment in part 2 correlated with any of the three
#    speeds of rating from part 1?

# For this, we just run a few correlations
with(epdat,cor.test(authenticity_RateRT,EmotionRT))
with(epdat,cor.test(intensity_RateRT,EmotionRT))
with(epdat,cor.test(trustworthiness_RateRT,EmotionRT))

# Nothing there... (and even if you found anything, remember you're running 3 tests!)

# b) I want to make sure that the emotional manipulation worked - do people rate angry faces as more intense?

# First let's graph this
# We need to aggregate data for ggplot
intagg = ddply(epdat,c('Emotion','OASSide','OASTime'),summarize,AvgRate=mean(intensity_Rating),SERate=se(intensity_Rating),
               AvgRT=mean(intensity_RateRT),SERT=se(intensity_RateRT))

ggplot(intagg,aes(x=OASSide,y=AvgRate,ymin=AvgRate-2*SERate,ymax=AvgRate+2*SERate,group=OASTime,color=OASTime)) + 
  geom_line() + geom_pointrange() + facet_grid(.~Emotion)

# Looks pretty clear
# What do our statistics say?
summary(aov(intensity_Rating ~ Emotion*OASSide*OASTime + Error(Subject/(Emotion*OASSide*OASTime)),data=epdat))

# The only thing here is a main effect. Good to know

# c) Are they faster to rate angry faces as intense than happy faces?

# Again, let's do the plot
ggplot(intagg,aes(x=OASSide,y=AvgRT,ymin=AvgRT-2*SERT,ymax=AvgRT+2*SERT,group=OASTime,color=OASTime)) + 
  geom_line() + geom_pointrange() + facet_grid(.~Emotion)

# Well this looks odd... stats?
summary(aov(intensity_RateRT ~ Emotion*OASSide*OASTime + Error(Subject/(Emotion*OASSide*OASTime)),data=epdat))

# It seem that there are a whole host of statistically significant and marginally significant things
# But they all seem to be driven by the 20ms left happy assymmetry being slower
# Is this just driven by outliers?
ggplot(epdat,aes(x=OASSide,y=intensity_RateRT,color=OASTime)) + geom_boxplot() + facet_grid(.~Emotion)

# Well maybe... but it does seem like there are a number of slower responses in that condition, not just extremes
# Nonetheless, it's not clear that angry is any faster than happy
# So we might think of that one condition as an oddity, but that's exploratory, not hypothesis driven

#######################
# Qualtrix (courtesy of Jess Thierman, from prior work with Kevin & Ed)

# This data came from an experiment investigating whether the Fundamental Attribution Error was moderated by ratings of intelligence
# The question is whether people are less likely to make attributions (or make them in the opposite direction)
#  if they thought that people were "tanking" the question
# For instance, if a smart person argues something with weak evidence, then they might not really hold that opinion

# Subjects read 8 'test responses' that people supposedly made as part of a debate class
# All of the 'test questions' were politically charged topics
# Subjects were told that the test takers had their side assigned to them (e.g., not their own political leanings)
# Answers were either Democratic or Republican, but also split by two other factors:
# They could be high or low support ('death penalty is bad since it is not shown to reduce crime' vs 'killing is just bad')
# And written intelligently or dumbly (lots of grammatical mistakes, etc.)
# For each of these 'tests', subjects rated both how intelligent they thought the test taker to be, and the test takers political leaning
# These ratings were on a scale from 0-100 (higher is smarter / more conservative)

# So, the conditions are marked by a three letter code, where each letter stands for:
#  1) D(emocratic) or R(epublican) argument
#  2) H(igh) or L(ow) support
#  3) S(mart) or D(umb) writing

# So RHS would be a Republican argument with good support, written intelligently
# And thus RHS_Int_1 is the rating of the RHS test taker's intelligence
# And RHS_Pol_1 is the rating of that test taker's political leanings
# But oops! Someone made a typo in Qualtrix and instead of DLS, they put in DHL
# So something is mislabeled here... and you need to fix it
# And if you look at the data, there are two header lines... and the second one seems worthless

# The main analysis will look at how intelligence and support of the argument affect percieved political attitudes and intelligence
# So your analyses should generally include those and political leaning where appropriate

# Finally, you will need to answer the following questions:
# a) Did our intelligence manipulation work? E.g., do people rate "smart" exams higher on intelligence than "dumb" exams?
# b) Did our political maipulation work? E.g., are "Republican" arguments rated more conservative?
# c) Do we have evidence that political attributions are specifically attenuated when smart people give low support arguments?

# Now since we want to run a repeated measures ANOVA on this, what we want is a data frame with the following collumns:
#  1) Subject ID
#  2) Democratic/Republican answers condition
#  3) High/Low support condition
#  4) Smart/Dumb writing condition
#  5) Subject rating of intelligence for those responses
#  6) Subject rating of policical leaning for those responses

# Qualtrix for the most part gives you an easy csv file: everything is filled and it's in nice columns
# It's in wide format, but we learned how to deal with that earlier
# There is really only one really tricky part to this file:
# We want to skip the second line

# To solve this problem, there are two solutions:
#  1) Open the .csv file in Excel and just delete the second row
#  2) Use R - this is trickier, but we can appeal to the internet
#     Google the phrase: "skip second line reading table r" and look at the first hit

# Since this is an R lesson, we'll use option #2
qualt_txt = textConnection(readLines(url('http://vulstats.ucsd.edu/RAssignData/DataCleaning_Qualtrix.csv'))[-2])
qualt = read.csv(qualt_txt)

# Now if we just use this data, there's going to be a lot of it. Like 101 columns worth
# But we only care about a small portion of this - we can ignore most of the timing and filler responses
# Qualtrix gives you a response for EVERYTHING, and that's unweildy
# So now we will pull out only the collumns we care about:
# The subject identifier and the response about political beliefs and intelligence for each condition
# (Note that if we were really dealing with this data, we would keep much more in - but this example can be extended)
# We will do that by naming the columns we care about

redqual = qualt[c('V1','DHS_Int_1','DHS_Pol_1','DHL_Int_1','DHL_Pol_1','DHD_Int_1','DHD_Pol_1','DLD_Int_1','DLD_Pol_1',
                'RHS_Int_1','RHS_Pol_1','RLS_Int_1','RLS_Pol_1','RHD_Int_1','RHD_Pol_1','RLD_Int_1','RLD_Pol_1')]

# Now we might want to replace the names to make it look nicer and get rid of the data entry error
# The easiest way is probably just to rewrite each of the names, rather than automate taking out the _1 and replaceing the DHL
names(redqual) = c('Subj','DHS_Int','DHS_Pol','DLS_Int','DLS_Pol','DHD_Int','DHD_Pol','DLD_Int','DLD_Pol',
                   'RHS_Int','RHS_Pol','RLS_Int','RLS_Pol','RHD_Int','RHD_Pol','RLD_Int','RLD_Pol')

# Next we want to get rid of the data that wasn't completely filled
# One issue with Qualtrix is that it's online - so if people quit midway through, you don't get complete data
# Often in these cases you'll just want to delete any rows missing this data (to be sure of balanced designs)
# Luckily, we have the na.omit() function which takes in a data frame and clears out all rows with any NA in them
redqual = na.omit(redqual)

# Okay - now this data is clean...
# But we need it in long format, not wide
# We can reshape the data though

longqual = melt(redqual,id.vars='Subj',variable.name='Test',value.name='Rating')
  
# Now we need to break this out the condition into the various types to make it factorial
# The good news is that each condition type is easy to find in the 'time' column...
# The political leaning is the first character, support amount the second, and so on
# So we use the substr() command that pulls out parts of character strings to segment these
longqual$Politics = factor(substr(longqual$Test,1,1))
head(longqual)

# Then I can do the same thing for all other conditions
longqual$Support = factor(substr(longqual$Test,2,2))
longqual$Intel = factor(substr(longqual$Test,3,3))
longqual$RateType = factor(substr(longqual$Test,5,7))
longqual = longqual[-2] # To remove the 'Test' column now we've split it up
head(longqual)

# Then we pull out ratings of intelligence and political leaning
intqual = subset(longqual, RateType == 'Int')
polqual = subset(longqual, RateType == 'Pol')

# And aggregate for viewing
intagg = ddply(intqual,c('Politics','Support','Intel'),summarize,AvgInt=mean(Rating),SEInt=se(Rating))
polagg = ddply(polqual,c('Politics','Support','Intel'),summarize,AvgPol=mean(Rating),SEPol=se(Rating))
qualagg = merge(polagg,intagg)

# Now let's look at our data:
ggplot(qualagg,aes(x=Intel,y=AvgPol,ymin=AvgPol-2*SEPol,ymax=AvgPol+2*SEPol,group=Support,color=Support)) +
  geom_line() + geom_pointrange() + facet_grid(.~Politics)

ggplot(qualagg,aes(x=Intel,y=AvgInt,ymin=AvgInt-2*SEInt,ymax=AvgInt+2*SEInt,group=Support,color=Support)) +
  geom_line() + geom_pointrange() + facet_grid(.~Politics)

# Let's answer our questions now:
# a) Did our intelligence manipulation work? E.g., do people rate "smart" exams higher on intelligence than "dumb" exams?

# Well the graph looks like it...
# But we can run a statistical test:

summary(aov(Rating ~ Politics*Support*Intel + Error(Subj / (Politics*Support*Intel)), data = intqual))

# Seems like there's clearly a difference by Intel

# But there are also some other differences...
# There's also a difference by support... so even intelligently written things with low support are seen
#  to be written by unintelligent people
# (That's not good for our analysis)

# And there are some other interactions (Politics:Support and the three-way) that exist
# These really shouldn't be there, and might be an effect of the stimuli we used

# b) Did our political maipulation work? E.g., are "Republican" arguments rated more conservative?

# Again the graph makes it clear
# But again let's run a test

summary(aov(Rating ~ Politics*Support*Intel + Error(Subj / (Politics*Support*Intel)), data = polqual))

# Again clear evidence that our manipulation worked
# There's also evidence of an interaction between Politics and Intel...
# Looking at the graph, it seems like Smart Republican arguments are rated as more conservative than Dumb Republican
# But the same pattern doesn't hold for Democrats
# This might be noise, but again might be an effect of our specific stimuli

# c) Do we have evidence that political attributions are specifically attenuated when smart people give low support arguments?

# Here we will need to control for differences between D and R
# Since they go in opposite directions, we can collapse over all of them
# Going 10 points below 50 for D is the same strength of FAE as going 10 points above 50 for R
# So we make normalized ratings of:

polqual$NormRating = with(polqual,ifelse(Politics=='D',50-Rating,Rating-50))

# We can again look at this:
normpolagg = ddply(polqual,c('Support','Intel'),summarize,AvgNPol=mean(NormRating),SENPol=se(NormRating))
ggplot(normpolagg,aes(x=Intel,y=AvgNPol,group=Support,color=Support,ymin=AvgNPol-2*SENPol,ymax=AvgNPol+2*SENPol)) +
  geom_line() + geom_pointrange()

# And do statistical analysis
# We can look at normalized rating as a function of Support and Intel
# If there is this modulation, then we should see an interaction
summary(aov(NormRating ~ Support*Intel + Error(Subj/(Support*Intel)),data=polqual))

# But we don't
# So we don't find evidence that people do reason about political leanings the way we thought they would